
function quot(str)
{
	if (!str)
		return '""';
		
	if (str.indexOf(" ") == -1)
		return str;
	return '{' + str + '}';
}

function updateupdown()
{
	$('#ruleset a.uprule').enable().filter(':first').disable();
	$('#ruleset a.downrule').enable().filter(':last').disable();
}

function update_buttons(changed)
{
	if (changed)
	{
		$('#b_save,#b_revert').enable();
		$('#pendingnote').show();
		$('#b_save')
		    .jConfirmAction({
			question: 'Save changes?',
			yesAnswer: 'Yes',
			cancelAnswer: 'No'
			}, function(el) {
				$('.jcaquestion').fadeOut(300);
				$.post('save.jim', {
					dir: $('span.dir').text(),
					data: conffile()
				}, function(data) {
					$('#output').html(data)
					   .delay(5000).fadeOut('slow');
					update_buttons(0);
				});
			});
		$('#b_revert')
		    .jConfirmAction({
			question: 'Discard changes?',
			yesAnswer: 'Yes',
			cancelAnswer: 'No'
			}, function(el) {
				$('.jcaquestion').fadeOut(300);
				loadrules($('span.dir').text());
				update_buttons(0);
			});
	}
	else
	{
		$('#b_save,#b_revert').disable();
		$('#pendingnote').hide();
	}
}

function update_rulehooks(obj)
{
	if (!obj)
		obj = $('#ruleset');

	obj.find('a.delclause').jConfirmAction({
		question: 'Delete criterion?',
		yesAnswer: 'Yes',
		cancelAnswer: 'No'
	}, function(el) {
		$('.jcaquestion').fadeOut(300);
		var rule = $(el).closest('div.rule');
		$(el).closest('tr.clause').fadeOut('slow', function() {
			$(this).remove();
			rawrefresh(rule);
			update_buttons(1);
		});
	});

	obj.find('a.editclause').on('click', function(e) {
		e.preventDefault();
		edit_clause($(this).closest('tr.clause'));
	});

	obj.find('button.addcriterion')
	    .button({icons: { primary: "ui-icon-plus"}})
	    .on('click', function(e) {
		e.preventDefault();
		var rule = $(this).closest('div.rule');
		edit_select(rule, 'Select new criterion',
		    select_criteria, '', function(rule, val) {
			var id = rule.attr('id');
			rule.find('table.criteria tbody')
			    .append(criterion({
				cmd: val,
				arg: schema.criterion[val].def
			    }));
			update_rulehooks(rule);
			update_buttons(1);
		});
	});

	obj.find('a.delrule').jConfirmAction({
		question: 'Delete entire rule?',
		yesAnswer: 'Yes',
		cancelAnswer: 'No'
	}, function(el) {
		$('.jcaquestion').fadeOut(300);
		$(el).closest('div.rule').slideUp('slow', function() {
			$(this).remove();
			update_buttons(1);
			updateupdown();
			if ($('div.rule').length < 1)
				$('#ruleset').attr('empty', true)
				    .html('<i>No rules defined for this ' +
				    'folder.</i>');
		});
	});

	obj.find('a.editcomment').on('click', function(e) {
		e.preventDefault();
		edit_text($(this), 'Edit rule name',
		    $(this).siblings('span.comment').text(),
		    function(obj, text) {
			$(obj).siblings('span.comment').text(text);
			update_buttons(1);
		});
	});

	obj.find('a.uprule').on('click', function(e) {
		e.preventDefault();
		if ($(this).hasClass('ui-state-disabled'))
			return false;
		var rule = $(this).closest('div.rule');
		var target = $(rule).prev('div.rule');
		if (!target || !target.length)
			return false;
		$(rule).addClass('hl').slideUp('slow', function() {
			$(this).insertBefore($(target))
			    .slideDown('slow', function() {
				$(this).removeClass('hl');
				updateupdown();
				update_buttons(1);
			    });
		});
	});

	obj.find('a.downrule').on('click', function(e) {
		e.preventDefault();
		if ($(this).hasClass('ui-state-disabled'))
			return false;
		var rule = $(this).closest('div.rule');
		var target = $(rule).next('div.rule');
		if (!target || !target.length)
			return false;
		$(rule).addClass('hl').slideUp('slow', function() {
			$(this).insertAfter($(target))
			    .slideDown('slow', function() {
				$(this).removeClass('hl');
				updateupdown();
				update_buttons(1);
			    });
		});
	});

	obj.find('a.editaction').on('click', function(e) {
		e.preventDefault();

		var rule = $(this).closest('div.rule');
		var type = rule.attr('type');

		if (type == 'folder')
			options = select_folder_actions;
		else
			options = select_file_actions;

		$('#edit_action_act').empty();
		$.each(options, function(k, v) {
			$('#edit_action_act').append(
				$('<option></option>').attr('value', k).text(v)
			);
		});

		var cmd = rule.find('tr.action th').attr('cmd');
		var arg = rule.find('tr.action td.val').html();

		if (schema.action[cmd].argtype == 'none')
			arg = '';

		if (schema.action[cmd].argtype == 'folder' &&
		    arg == 'Media Root')
			arg = '';

		$('#edit_action_act').val(cmd);
		$('#edit_action_arg').val(arg);

		$('#edit_action').dialog({
			height: 'auto', width: 'auto',
			draggable: false, resizable: false,
			autoOpen: true,
			position: {
				my: 'bottom left',
				at: 'top right',
				of: rule
			},
			buttons: {
			    "Save": function() {
				cmd = $('#edit_action_act').val();
				arg = $('#edit_action_arg').val();
				
				rule.find('tr.action').replaceWith(
				    action({cmd: cmd, arg: arg}));
				$(this).dialog('close');
				rawrefresh(rule);
				update_rulehooks(rule);
				update_buttons(1);
			    },
			    "Close": function() {
				$(this).dialog('close');
			    }
			}
		});
	});
}

var setters = {
	int: function(cmd, a) {
		var b = a.split(" ");
		if (b.length == 1)
		{
			op = '=';
			val = a;
		}
		else
		{
			op = b[0];
			val = b[1];
		}
		return op + ' ' + val;
	},
	substr: function(cmd, a) {
		return a;
	},
	select: function(cmd, a) {
		return schema.criterion[cmd].select[a];
	}
};

var getters = {
	int: function(cmd, a) {
		var b = a.split(" ");
		if (b.length != 2)
			return a;

		if (b[0] == '=')
			return b[1];
		return a;
	},
	substr: function(cmd, a) {
		return a;
	},
	select: function(cmd, a) {
		var c = schema.criterion[cmd].select;
		for (key in c)
			if (c[key] == a)
				return key;
		return 'UNKNOWN (' + a + ')';
	}
};

function ruleconf(rule)
{
	var s = '';

	if ($(rule).attr('type') == 'folder')
		s += 'folder ';

	$(rule).find('tr.clause').each(function(i) {
		cmd = $(this).find('th').attr('cmd');
		c = schema.criterion[cmd];
		val = $.trim($(this).find('td').text());
		if (getters[c.type])
			val = getters[c.type](cmd, val);

		s += cmd + ' ' + quot(val) + ' ';
	});

	act = $(rule).find('tr.action th').attr('cmd');
	arg = $.trim($(rule).find('tr.action td').text());
	if (schema.action[act].argtype == 'folder' && arg == 'Media Root')
		arg = '';
	if (act != 'continue')
		s += 'action {' + act + ' ' + quot(arg) + '}';
	return s;
}

function conffile()
{
	var s = '';

	$('div.rule').each(function(i) {
		comment = $(this).find('span.comment').text();
		if (comment != 'Unnamed rule')
			s += '# ' + comment + '\n';
		s += ruleconf($(this)) + '\n';
	});
	return s;
}

function rawrefresh(rule)
{
	$(rule).find('.raw').html(ruleconf(rule));
}

function criterion(data)
{
	var c = schema.criterion[data.cmd];
	var s;

	s = '<tr class=clause>' +
	    '<th cmd=' + data.cmd + '>' + c.desc + '</th><td class=val>';

	if (setters[c.type])
		s += setters[c.type](data.cmd, data.arg);
	else
		s += 'UNKNOWN (' + data.arg + ')';

	s += '</td><td>' +
	    '<a class=editclause href=#>' +
	    '<img src=/img/context/edit.png></a>' +
	    '&nbsp;' +
	    '<a class=delclause href=#>' +
	    '<img src=/img/context/delete.png></a>' +
	    '</td></tr>';
	return s;
}

function action(data)
{
	var c = schema.action[data.cmd];
	var s;

	if (c.argtype == 'folder' && !data.arg)
		data.arg = 'Media Root';

	s = '<tr class=action>' +
	    '<th cmd=' + data.cmd + '>' + c.desc + '</th>' +
	    '<td class=val>' + data.arg + '</td>'  +
	    '<td><a class=editaction href=#>' +
	    '<img src=/img/context/edit.png></a></td>' +
	    '</tr>';

	return s;
}

var last_ruleid = 0;

function addrule(id, data)
{
	if ($('#ruleset').attr('empty'))
		$('#ruleset').empty().removeAttr('empty');
	last_ruleid = id;
	var rule = $('#rule_template')
		.clone()
		.attr('id', 'rule_' + id)
		.addClass('rule')
		.removeClass('hidden');

	// Only Firefox seems to have problems with top-right button
	// placement.
	if (navigator.userAgent.indexOf("Firefox") != -1)
		rule.find('span.legendright').css('top', '-20px');

	rule.find('span.comment').html(data.name);
	if (data.type == 'folder')
		rule.find('.commentleft > span.folder').show();
	else
		rule.find('.commentleft > span.file').show();
	rule.attr('type', data.type);

	var $c = rule.find('table.criteria');
	$.each(data.criteria, function(key, val) {
		$($c).find('tbody').append(criterion(val));
	});

	rule.find('table.action tbody').append(action(data.action));

	rule.find('.raw').html(data.raw);

	$('#ruleset').append(rule);

	return rule;
}

function edit_text(obj, title, text, callback)
{
	$('#edit_text_field').val(text);
	$('#edit_text').dialog({
		title: title,
		height: 'auto', width: 'auto',
		draggable: false, resizable: false,
		autoOpen: true,
		position: {
			my: 'bottom left',
			at: 'top right',
			of: obj
		},
		buttons: {
			"Save": function() {
				var val = $.trim($('#edit_text_field').val());
				if (!val)
					alert("Bad value");
				else
				{
					callback(obj, val);
					$(this).dialog('close');
				}
			},
			"Close": function() {
				$(this).dialog('close');
			}
		}
	});
}

function edit_int(obj, title, op, val, callback)
{
	$('#edit_int_field').val(val);
	$('#edit_int_op').val(op);
	$('#edit_int').dialog({
		title: title,
		height: 'auto', width: 'auto',
		draggable: false, resizable: false,
		autoOpen: true,
		position: {
			my: 'bottom left',
			at: 'top right',
			of: obj
		},
		buttons: {
			"Save": function() {
				var op = $.trim($('#edit_int_op').val());
				var val = $.trim($('#edit_int_field').val());
				if (!op || !val)
					alert("Bad value");
				else
				{
					callback(obj, op, val);
					$(this).dialog('close');
				}
			},
			"Close": function() {
				$(this).dialog('close');
			}
		}
	});
}

function edit_select(obj, title, options, val, callback)
{
	$('#edit_select_field').empty();
	$.each(options, function(k, v) {
		$('#edit_select_field').append(
			$('<option></option>').attr('value', k).text(v)
		);
	});
	if (val && val != '')
		$('#edit_select_field').val(val);
	$('#edit_select').dialog({
		title: title,
		height: 'auto', width: 'auto',
		draggable: false, resizable: false,
		autoOpen: true,
		position: {
			my: 'bottom left',
			at: 'top right',
			of: obj
		},
		buttons: {
			"Save": function() {
				callback(obj, $('#edit_select_field').val());
				$(this).dialog('close');
			},
			"Close": function() {
				$(this).dialog('close');
			}
		}
	});
}

function edit_clause(obj)
{
	var cmd = $(obj).find('th').attr('cmd');
	var target = $(obj).find('td.val');
	var val = $.trim($(target).text());
	var title = $(obj).find('th').html();
	if (!schema.criterion[cmd])
	{
		alert('Unhandled command (' + cmd + ')');
		return;
	}
	switch (schema.criterion[cmd].type)
	{
	    case 'substr':
		edit_text(target, title, val, function(obj, text) {
			$(obj).text(text);
			rawrefresh($(obj).closest('div.rule'));
			update_buttons(1);
		});
		break;
	    case 'int':
		b = val.split(" ");
		if (b.length != 2)
		{
			alert('Bad int value');
			break;
		}
		edit_int(target, title, b[0], b[1], function(obj, op, val) {
			$(obj).text(op + ' ' + val);
			rawrefresh($(obj).closest('div.rule'));
			update_buttons(1);
		});
		break;
	    case 'select':
		edit_select(target, title, schema.criterion[cmd].select,
		    getters.select(cmd, val), function(obj, val) {
			$(obj).text(setters.select(cmd, val));
			rawrefresh($(obj).closest('div.rule'));
			update_buttons(1);
		});
		break;
		
	    default:
		alert('Unhandled clause type (' + cmd + ')');
	}
}

function loadrules(dir)
{
    $('#ruleset')
	.html('<img src=/img/loading.gif> Loading rules...');
    $.getJSON('rules_json.jim', 'dir=' + dir, function(data) {
	$('#ruleset').empty();
	if (!data.length)
	{
		$('#ruleset').attr('empty', true)
		    .html('<i>No rules defined for this folder.</i>');
		return;
	}
	$.each(data, function(key, val) {
		addrule(key, val);
	});
	updateupdown();
	if (!showraw)
		$('#ruleset div.raw').hide();

	update_rulehooks();
    });
}

$(function() {

loadrules($('span.dir').text());
$('#b_add').button({icons: {primary: "ui-icon-plus"}})
    .on('click', function(e) {
	e.preventDefault();
	$('#newrule').dialog({
		height: 'auto', width: 'auto',
		draggable: false, resizable: false,
		autoOpen: true,
		position: {
			my: 'bottom left',
			at: 'top right',
			of: $(this)
		},
		buttons: {
			"Create Rule": function() {
				rule = addrule(last_ruleid + 1, {
					raw: '',
					name: $('#newrule_comment').val(),
					type: $('input[name=newrule_type]' +
					    ':checked').val(),
					criteria: [],
					action: {
						cmd: 'continue',
						arg: ''
					}
				});
				update_rulehooks(rule);
				update_buttons(1);
				$(this).dialog('close');
			},
			"Close": function() {
				$(this).dialog('close');
			}
		}
	});
	$('#newrule_type_file').prop('checked', true);
	$('#newrule_comment').focus().val('').val('Unnamed rule');
});
$('#b_save').button({icons: {primary: "ui-icon-disk"}});
$('#b_revert').button({icons: {primary: "ui-icon-arrowrefresh-1-w"}});
$('#b_show').button({icons: {primary: "ui-icon-clipboard"}})
    .on('click', function(e) {
	e.preventDefault();
	$('#showconf')
	    .empty()
	    .html('<pre>' + conffile() + '</pre>')
	    .dialog({
		height: 'auto', width: 'auto',
		autoOpen: true,
		modal: true,
		buttons: {
			"Close": function() {
				$(this).dialog('close');
			}
		}
	});
});
$('#b_raw').button({icons: {primary: "ui-icon-gear"}})
    .on('click', function(e) {
	$('div.raw').toggle('slow');
});
update_buttons(0);

});

